from enum import Enum


class ScenarioType(str, Enum):
    COMMON_CONTRACTIONS = "COMMON_CONTRACTIONS"
    COMMON_MISSPELLINGS = "COMMON_MISSPELLINGS"
    CONDITIONAL = "CONDITIONAL"
    LABEL_REVERSE_INVARIANT = "LABEL_REVERSE_INVARIANT"
    NAMED_ENTITY_SUBSTITUTION = "NAMED_ENTITY_SUBSTITUTION"
    NEGATION = "NEGATION"
    REPHRASE_INVARIANT = "REPHRASE_INVARIANT"
    ROUNDTRIP_INVARIANT = "ROUNDTRIP_INVARIANT"
    SEED = "SEED"
    TERM_RELEVANCE_INVARIANT = "TERM_RELEVANCE_INVARIANT"
    TEXT_REVERSE_LABELED = "TEXT_REVERSE_LABELED"
    TEXT_REVERSE_QUESTION = "TEXT_REVERSE_QUESTION"
    WORD_CHARACTER_INSERTION = "WORD_CHARACTER_INSERTION"
    WORD_INFLECTIONS = "WORD_INFLECTIONS"
    WORD_QWERTY_MISSPELL = "WORD_QWERTY_MISSPELL"
    WORD_SYNONYM_EMBEDDING = "WORD_SYNONYM_EMBEDDING"

    def __str__(self) -> str:
        return str(self.value)
